import 'package:flutter/material.dart';

import 'package:intl/intl.dart';
import 'package:provider/provider.dart';
import 'package:senior_design_tokens/senior_design_tokens.dart';

import '../../../repositories/theme_repository.dart';
import '../senior_calendar_style.dart';

typedef void OnDayLongPressed(DateTime day);

class SeniorCalendarDay extends StatelessWidget {
  const SeniorCalendarDay({
    Key? key,
    this.disableDayPressed = false,
    this.disableNextDayMonthPressed = false,
    this.disablePrevDayMonthPressed = false,
    this.disableWeekendPressed = false,
    this.disableYesterdayPressed = false,
    required this.firstDayOfWeek,
    this.highlightToday = true,
    this.inactiveYesterdays = true,
    required this.index,
    required this.isNextMonthDay,
    required this.isPrevMonthDay,
    required this.isSelectable,
    required this.isSelectedDay,
    required this.isThisMonthDay,
    required this.isToday,
    required this.isWeekendDay,
    required this.isYesterday,
    required this.localeDate,
    required this.now,
    required this.onDayPressed,
    this.onDayLongPressed,
    this.style,
  }) : super(key: key);

  final bool disableDayPressed;
  final bool disableNextDayMonthPressed;
  final bool disablePrevDayMonthPressed;
  final bool disableWeekendPressed;
  final bool disableYesterdayPressed;
  final int firstDayOfWeek;
  final bool inactiveYesterdays;
  final int index;
  final bool isNextMonthDay;
  final bool isPrevMonthDay;
  final bool isSelectable;
  final bool isSelectedDay;
  final bool isThisMonthDay;
  final bool isToday;
  final bool isWeekendDay;
  final bool isYesterday;
  final bool highlightToday;
  final DateFormat localeDate;
  final DateTime now;
  final OnDayLongPressed? onDayLongPressed;
  final Function(DateTime)? onDayPressed;
  final SeniorCalendarStyle? style;

  void _onDayPressed({
    required DateTime picked,
    required bool isWeekendDay,
    required bool isYesterday,
    required bool isPrevMonthDay,
    required bool isNextMonthDay,
  }) {
    if (disableDayPressed ||
        disableYesterdayPressed && isYesterday ||
        disableWeekendPressed && isWeekendDay ||
        disableNextDayMonthPressed && isNextMonthDay ||
        disablePrevDayMonthPressed && isPrevMonthDay) {
      return;
    }

    onDayPressed?.call(picked);
  }

  void _onDayLongPressed(DateTime picked) {
    onDayLongPressed?.call(picked);
  }

  Color _getDayColor(BuildContext context) {
    final theme = Provider.of<ThemeRepository>(context).theme;

    final Color todayColor = style?.todayColor ??
        theme.calendarTheme?.style?.todayColor ??
        SeniorColors.neutralColor800;

    final Color selectedDayColor = style?.selectedDayColor ??
        theme.calendarTheme?.style?.selectedDayColor ??
        SeniorColors.neutralColor800;

    final previousDaysMonthColor = style?.previousMonthDaysColor ??
        theme.calendarTheme?.style?.previousMonthDaysColor ??
        SeniorColors.neutralColor500;

    final nextDaysMonthColor = style?.nextMonthDaysColor ??
        theme.calendarTheme?.style?.nextMonthDaysColor ??
        SeniorColors.neutralColor500;

    final daysColor = style?.dayColor ??
        theme.calendarTheme?.style?.dayColor ??
        SeniorColors.neutralColor800;

    if (isSelectedDay) {
      return isToday ? todayColor : selectedDayColor;
    } else if (isToday) {
      return todayColor;
    } else if (isPrevMonthDay) {
      return previousDaysMonthColor;
    } else if (isNextMonthDay) {
      return nextDaysMonthColor;
    } else {
      return daysColor;
    }
  }

  Color _getBackgroundColor(BuildContext context) {
    final theme = Provider.of<ThemeRepository>(context).theme;

    final todayColor = style?.todayBackgroundColor ??
        theme.calendarTheme?.style?.todayBackgroundColor ??
        SeniorColors.neutralColor200;

    final previousMonthDayColor = style?.previousMonthDaysBackgroundColor ??
        theme.calendarTheme?.style?.previousMonthDaysBackgroundColor ??
        SeniorColors.neutralColor500;

    final nextMonthDayColor = style?.nextMonthDaysBackgroundColor ??
        theme.calendarTheme?.style?.nextMonthDaysBackgroundColor ??
        SeniorColors.neutralColor500;

    final selectedDayColor = style?.selectedDayBackgroundColor ??
        theme.calendarTheme?.style?.selectedDayBackgroundColor ??
        SeniorColors.primaryColor100;

    if (isSelectedDay) {
      return isToday && highlightToday ? todayColor : selectedDayColor;
    } else if (isToday && highlightToday) {
      return todayColor;
    } else if (isPrevMonthDay) {
      return previousMonthDayColor;
    } else if (isNextMonthDay) {
      return nextMonthDayColor;
    } else {
      return Colors.transparent;
    }
  }

  @override
  Widget build(BuildContext context) {
    final dayColor = _getDayColor(context);

    final dayComponent = GestureDetector(
      onLongPress: () => _onDayLongPressed(now),
      child: TextButton(
        style: TextButton.styleFrom(
          padding: const EdgeInsets.all(SeniorSpacing.xxxsmall),
          backgroundColor: _getBackgroundColor(context),
          shape: const CircleBorder(),
        ),
        onPressed: () => _onDayPressed(
          picked: now,
          isWeekendDay: isWeekendDay,
          isYesterday: isYesterday,
          isPrevMonthDay: isPrevMonthDay,
          isNextMonthDay: isNextMonthDay,
        ),
        child: Container(
          child: Center(
            child: Text(
              '${now.day}',
              semanticsLabel: now.day.toString(),
              style: SeniorTypography.label(color: dayColor),
              maxLines: 1,
            ),
          ),
        ),
      ),
    );

    return Container(
      margin: const EdgeInsets.symmetric(vertical: SeniorSpacing.small / 2),
      child: dayComponent,
    );
  }
}
