import 'dart:async';

import 'package:flutter/material.dart';

import 'package:intl/date_symbol_data_local.dart';
import 'package:intl/intl.dart' show DateFormat;
import 'package:provider/provider.dart';
import 'package:senior_design_tokens/senior_design_tokens.dart';

import 'components/senior_calendar_footer.dart';
import 'components/senior_calendar_header.dart';
import 'components/senior_weekday_row.dart';
import 'components/senior_calendar_day.dart';
import 'models/senior_event_calendar.dart';
import '../../components/senior_calendar/senior_calendar.dart';
import '../../repositories/repositories.dart';

typedef MarkedDateIconBuilder<T> = Widget? Function(T event);
typedef void OnDayLongPressed(DateTime day);

/// This builder is called for every day in the calendar.
/// If you want to build only few custom day containers, return null for the days you want to leave with default looks
/// All characteristics like circle border are also applied to the custom day container [DayBuilder] provides.
/// (if supplied function returns null, Calendar's function will be called for [day]).
/// [isSelectable] - is between [SeniorCalendar.minSelectedDate] and [SeniorCalendar.maxSelectedDate]
/// [index] - DOES NOT equal day number! Index of the day built in current visible field
/// [isSelectedDay] - if the day is selected
/// [isToday] - if the day is similar to [DateTime.now()]
/// [isPrevMonthDay] - if the day is from previous month
/// [textStyle] - text style that would have been applied by the calendar if it was to build the day.
/// Example: if the user provided [SeniorCalendar.todayTextStyle] and [isToday] is true,
///   [SeniorCalendar.todayTextStyle] would be sent into [DayBuilder]'s [textStyle]. If user didn't
///   provide it, default [SeniorCalendar]'s textStyle would be sent. Same applies to all text styles like
///   [SeniorCalendar.prevDaysTextStyle], [SeniorCalendar.daysTextStyle] etc.
/// [isNextMonthDay] - if the day is from next month
/// [isThisMonthDay] - if the day is from next month
/// [day] - day being built.
/// [isWeekendDay] - if the day is a weekend day.
typedef Widget? DayBuilder(bool isSelectable, int index, bool isSelectedDay, bool isToday, bool isPrevMonthDay,
    TextStyle textStyle, bool isNextMonthDay, bool isThisMonthDay, bool isWeekendDay, DateTime day);

/// This builder is called for every weekday container (7 times, from Mon to Sun).
/// [weekday] - weekday built, from 0 to 6.
/// [weekdayName] - string representation of the weekday (Mon, Tue, Wed, etc).
typedef Widget WeekdayBuilder(int weekday, String weekdayName);

class SeniorCalendar<T extends EventCalendarInterface> extends StatefulWidget {
  SeniorCalendar({
    Key? key,
    this.bottomText,
    this.childAspectRatio = 1.0,
    this.customGridViewPhysics,
    this.customWeekDayBuilder,
    this.disableDayPressed = false,
    this.disableNextDayMonthPressed = false,
    this.disablePrevDayMonthPressed = false,
    this.disableWeekendPressed = false,
    this.disableYesterdayPressed = false,
    this.firstDayOfWeek = 0,
    this.headerText,
    this.headerTitleTouchable = false,
    this.highlightToday = true,
    this.inactiveWeekendAfterToday = true,
    this.inactiveWeekendBeforeToday = true,
    this.inactiveYesterdays = true,
    this.isScrollable = true,
    this.locale = 'pt_BR',
    this.maxSelectedDate,
    this.minSelectedDate,
    this.onCalendarChanged,
    this.onDayPressed,
    this.onDayLongPressed,
    this.onHeaderTitlePressed,
    this.onLeftArrowPressed,
    this.onRightArrowPressed,
    this.pageScrollPhysics = const BouncingScrollPhysics(),
    this.pageSnapping = false,
    this.scrollDirection = Axis.horizontal,
    this.selectedDateTime,
    this.shouldShowTransform = true,
    this.showHeader = true,
    this.showHeaderButton = true,
    this.showOnlyCurrentMonthDate = false,
    this.showWeekDays = true,
    this.staticSixWeekFormat = false,
    this.style,
    this.targetDateTime,
    this.topTextCalendar,
    this.viewportFraction = 1.0,
    this.weekFormat = false,
    TextStyle? selectedDayTextStyle,
  }) : super(key: key);

  final String? bottomText;
  final double childAspectRatio;
  final ScrollPhysics? customGridViewPhysics;
  final WeekdayBuilder? customWeekDayBuilder;
  final bool disableDayPressed;
  final bool disableNextDayMonthPressed;
  final bool disablePrevDayMonthPressed;
  final bool disableWeekendPressed;
  final bool disableYesterdayPressed;
  final int? firstDayOfWeek;
  final String? headerText;
  final bool headerTitleTouchable;
  final bool highlightToday;
  final bool inactiveWeekendAfterToday;
  final bool inactiveWeekendBeforeToday;
  final bool inactiveYesterdays;
  final bool isScrollable;
  final String locale;
  final DateTime? maxSelectedDate;
  final DateTime? minSelectedDate;
  final Function(DateTime)? onCalendarChanged;
  final Function(DateTime)? onDayPressed;
  final OnDayLongPressed? onDayLongPressed;
  final Function? onHeaderTitlePressed;
  final Function? onLeftArrowPressed;
  final Function? onRightArrowPressed;
  final DateTime? selectedDateTime;
  final ScrollPhysics pageScrollPhysics;
  final bool pageSnapping;
  final Axis scrollDirection;
  final bool shouldShowTransform;
  final bool showHeader;
  final bool showHeaderButton;
  final bool showOnlyCurrentMonthDate;
  final bool showWeekDays;
  final bool staticSixWeekFormat;
  final SeniorCalendarStyle? style;
  final DateTime? targetDateTime;
  final String? topTextCalendar;
  final double viewportFraction;
  final bool weekFormat;

  @override
  _SeniorCalendarState<T> createState() => _SeniorCalendarState<T>();

  void pickCalendar(BuildContext context) {
    showModalBottomSheet<void>(
      backgroundColor: SeniorColors.pureWhite,
      isScrollControlled: true,
      elevation: 1,
      shape: const RoundedRectangleBorder(
        borderRadius: BorderRadius.vertical(top: Radius.circular(SeniorSpacing.normal)),
      ),
      context: context,
      builder: (BuildContext context) {
        return SeniorCalendar(
          bottomText: bottomText,
          childAspectRatio: childAspectRatio,
          customGridViewPhysics: customGridViewPhysics,
          customWeekDayBuilder: customWeekDayBuilder,
          disableDayPressed: disableDayPressed,
          disableNextDayMonthPressed: disableNextDayMonthPressed,
          disablePrevDayMonthPressed: disablePrevDayMonthPressed,
          disableWeekendPressed: disableWeekendPressed,
          disableYesterdayPressed: disableYesterdayPressed,
          firstDayOfWeek: firstDayOfWeek,
          headerText: headerText,
          headerTitleTouchable: headerTitleTouchable,
          highlightToday: highlightToday,
          inactiveWeekendAfterToday: inactiveWeekendAfterToday,
          inactiveWeekendBeforeToday: inactiveWeekendBeforeToday,
          inactiveYesterdays: inactiveYesterdays,
          isScrollable: isScrollable,
          locale: locale,
          maxSelectedDate: maxSelectedDate,
          minSelectedDate: minSelectedDate,
          onCalendarChanged: onCalendarChanged,
          onDayLongPressed: onDayLongPressed,
          onDayPressed: onDayPressed,
          onHeaderTitlePressed: onHeaderTitlePressed,
          onLeftArrowPressed: onLeftArrowPressed,
          onRightArrowPressed: onRightArrowPressed,
          pageScrollPhysics: pageScrollPhysics,
          pageSnapping: pageSnapping,
          scrollDirection: scrollDirection,
          selectedDateTime: selectedDateTime,
          shouldShowTransform: shouldShowTransform,
          showHeader: showHeader,
          showHeaderButton: showHeaderButton,
          showOnlyCurrentMonthDate: showOnlyCurrentMonthDate,
          showWeekDays: showWeekDays,
          staticSixWeekFormat: staticSixWeekFormat,
          targetDateTime: targetDateTime,
          topTextCalendar: topTextCalendar,
          viewportFraction: viewportFraction,
          weekFormat: weekFormat,
        );
      },
    );
  }
}

class _SeniorCalendarState<T extends EventCalendarInterface> extends State<SeniorCalendar<T>>
    with SingleTickerProviderStateMixin {
  late final CALENDAR_HEIGHT;

  late PageController _controller;
  late List<DateTime> _dates;
  late List<List<DateTime>> _weeks;
  DateTime _selectedDate = DateTime.now();
  late DateTime _targetDate;
  int _startWeekday = 0;
  int _endWeekday = 0;
  late DateFormat _localeDate;
  int _pageNum = 0;
  late DateTime minDate;
  late DateTime maxDate;
  late AnimationController animationController = AnimationController(
      vsync: this, duration: const Duration(milliseconds: 500), reverseDuration: const Duration(milliseconds: 500));

  // When FIRSTDAYOFWEEK is 0 in dart-intl, it represents Monday. However it is the second day in the arrays of Weekdays.
  // Therefore we need to add 1 modulo 7 to pick the right weekday from intl. (cf. [GlobalMaterialLocalizations])
  late int firstDayOfWeek;

  // If the setState called from this class, don't reload the selectedDate, but it should reload selected date if called from external class
  @override
  void initState() {
    super.initState();
    initializeDateFormatting();

    minDate = widget.minSelectedDate ?? DateTime(2018);
    maxDate = widget.maxSelectedDate ?? DateTime(DateTime.now().year + 1, DateTime.now().month, DateTime.now().day);

    final selectedDateTime = widget.selectedDateTime;

    CALENDAR_HEIGHT = widget.bottomText != null ? 472.0 : 448.0;

    if (selectedDateTime != null) {
      _selectedDate = selectedDateTime;
    }

    _init();

    _controller = PageController(
      initialPage: _pageNum,
      keepPage: true,
      viewportFraction: widget.viewportFraction,
    );

    _localeDate = DateFormat('MMMM yyyy', widget.locale);
    firstDayOfWeek = widget.firstDayOfWeek ?? (_localeDate.dateSymbols.FIRSTDAYOFWEEK + 1) % 7;

    _setDate();
  }

  @override
  void didUpdateWidget(SeniorCalendar<T> oldWidget) {
    if (widget.targetDateTime != null && widget.targetDateTime != _targetDate) {
      _init();
      _setDate(_pageNum);
    }

    super.didUpdateWidget(oldWidget);
  }

  @override
  void dispose() {
    _controller.dispose();
    super.dispose();
  }

  void _init() {
    final targetDateTime = widget.targetDateTime;
    if (targetDateTime != null) {
      if (targetDateTime.difference(minDate).inDays < 0) {
        _targetDate = minDate;
      } else if (targetDateTime.difference(maxDate).inDays > 0) {
        _targetDate = maxDate;
      } else {
        _targetDate = targetDateTime;
      }
    } else {
      _targetDate = _selectedDate;
    }
    if (widget.weekFormat) {
      _pageNum = _targetDate.difference(_firstDayOfWeek(minDate)).inDays ~/ 7;
    } else {
      _pageNum = (_targetDate.year - minDate.year) * 12 + _targetDate.month - minDate.month;
    }
  }

  @override
  Widget build(BuildContext context) {
    final theme = Provider.of<ThemeRepository>(context).theme;

    return Container(
      padding: const EdgeInsets.all(SeniorSpacing.normal),
      decoration: BoxDecoration(
        borderRadius: const BorderRadius.vertical(
          top: Radius.circular(SeniorRadius.xxbig),
        ),
        color: widget.style?.backgroundColor ?? theme.calendarTheme?.style?.backgroundColor ?? SeniorColors.pureWhite,
      ),
      child: Container(
        height: CALENDAR_HEIGHT,
        child: Column(
          children: [
            Container(
              height: 4.0,
              width: 62.0,
              margin: const EdgeInsets.only(bottom: SeniorSpacing.normal),
              decoration: BoxDecoration(
                borderRadius: BorderRadius.circular(SeniorRadius.xxbig),
                color: widget.style?.topLineColor ??
                    theme.calendarTheme?.style?.topLineColor ??
                    SeniorColors.neutralColor300,
              ),
            ),
            widget.topTextCalendar != null
                ? Container(
                    width: double.infinity,
                    padding: const EdgeInsets.symmetric(horizontal: SeniorSpacing.normal),
                    child: Text(
                      widget.topTextCalendar!,
                      style: SeniorTypography.label(
                        color: widget.style?.topTextColor ??
                            theme.calendarTheme?.style?.topTextColor ??
                            SeniorColors.neutralColor700,
                      ),
                      textAlign: TextAlign.left,
                    ),
                  )
                : const SizedBox.shrink(),
            SeniorCalendarHeader(
              style: widget.style,
              showHeader: widget.showHeader,
              headerTitle: buildHeaderText(),
              showHeaderButtons: widget.showHeaderButton,
              leftButtonActive: _pageNum > 0,
              rightButtonActive: !(_dates.length == (_pageNum + 1)),
              onLeftButtonPressed: () {
                widget.onLeftArrowPressed?.call();

                if (_pageNum > 0) {
                  _setDate(_pageNum - 1);
                }
              },
              onRightButtonPressed: () {
                widget.onRightArrowPressed?.call();

                if (widget.weekFormat) {
                  if (_weeks.length - 1 > _pageNum) {
                    _setDate(_pageNum + 1);
                  }
                } else {
                  if (_dates.length - 1 > _pageNum) {
                    _setDate(_pageNum + 1);
                  }
                }
              },
              onHeaderTitlePressed: widget.headerTitleTouchable
                  ? () {
                      final onHeaderTitlePressed = widget.onHeaderTitlePressed;
                      if (onHeaderTitlePressed != null) {
                        onHeaderTitlePressed();
                      } else {
                        _selectDateFromPicker();
                      }
                    }
                  : null,
            ),
            SeniorWeekdayRow(
              firstDayOfWeek,
              widget.customWeekDayBuilder,
              showWeekdays: widget.showWeekDays,
              localeDate: _localeDate,
              style: widget.style,
            ),
            Expanded(
              child: PageView.builder(
                itemCount: widget.weekFormat ? _weeks.length : _dates.length,
                physics: widget.isScrollable ? widget.pageScrollPhysics : const NeverScrollableScrollPhysics(),
                onPageChanged: (index) => _setDate(index),
                controller: _controller,
                itemBuilder: (context, index) => widget.weekFormat ? weekBuilder(index) : builder(index),
                pageSnapping: widget.pageSnapping,
              ),
            ),
            SeniorCalendarFooter(
              text: widget.bottomText,
              style: widget.style,
            ),
          ],
        ),
      ),
    );
  }

  AnimatedBuilder builder(int slideIndex) {
    _startWeekday = _dates[slideIndex].weekday - firstDayOfWeek;
    if (_startWeekday == 7) {
      _startWeekday = 0;
    }
    _endWeekday = DateTime(_dates[slideIndex].year, _dates[slideIndex].month + 1, 1).weekday - firstDayOfWeek;
    final double screenWidth = MediaQuery.of(context).size.width;
    final int totalItemCount = widget.staticSixWeekFormat
        ? 42
        : DateTime(
              _dates[slideIndex].year,
              _dates[slideIndex].month + 1,
              0,
            ).day +
            _startWeekday +
            (7 - _endWeekday);

    final int year = _dates[slideIndex].year;
    final int month = _dates[slideIndex].month;

    return AnimatedBuilder(
      animation: _controller,
      builder: (context, child) {
        if (!widget.shouldShowTransform) {
          return child!;
        }
        double value = 1.0;
        if (_controller.position.haveDimensions) {
          value = _controller.page! - slideIndex;
          value = (1 - (value.abs() * .6)).clamp(0.75, 1.0);
        }

        return Center(
          child: SizedBox(
            height: Curves.easeOut.transform(value) * CALENDAR_HEIGHT,
            width: Curves.easeOut.transform(value) * screenWidth,
            child: child,
          ),
        );
      },
      child: Stack(
        children: <Widget>[
          Container(
            width: double.infinity,
            height: double.infinity,
            child: GridView.count(
              physics: widget.customGridViewPhysics,
              crossAxisCount: 7,
              childAspectRatio: 1.0,
              children: List.generate(
                totalItemCount,
                (index) {
                  final selectedDateTime = _selectedDate;

                  final isToday = DateTime.now().day == index + 1 - _startWeekday &&
                      DateTime.now().month == month &&
                      DateTime.now().year == year;

                  final isSelectedDay = selectedDateTime.year == year &&
                      selectedDateTime.month == month &&
                      selectedDateTime.day == index + 1 - _startWeekday;

                  final isPrevMonthDay = index < _startWeekday;
                  final isNextMonthDay = index >= (DateTime(year, month + 1, 0).day) + _startWeekday;
                  final isThisMonthDay = !isPrevMonthDay && !isNextMonthDay;

                  DateTime now = DateTime(year, month, 1);

                  if (isPrevMonthDay && !widget.showOnlyCurrentMonthDate) {
                    now = now.subtract(Duration(days: _startWeekday - index));
                  } else if (isThisMonthDay) {
                    now = DateTime(year, month, index + 1 - _startWeekday);
                  } else if (!widget.showOnlyCurrentMonthDate) {
                    now = DateTime(year, month, index + 1 - _startWeekday);
                  } else {
                    return Container();
                  }
                  bool isSelectable = true;
                  if (now.millisecondsSinceEpoch < minDate.millisecondsSinceEpoch)
                    isSelectable = false;
                  else if (now.millisecondsSinceEpoch > maxDate.millisecondsSinceEpoch) {
                    isSelectable = false;
                  }
                  final bool isYesterday = now.isBefore(DateTime.now()) && !isToday;
                  final bool isWeekendDay = now.weekday == DateTime.saturday || now.weekday == DateTime.sunday;

                  return SeniorCalendarDay(
                    disableDayPressed: widget.disableDayPressed,
                    disableNextDayMonthPressed: widget.disableNextDayMonthPressed,
                    disablePrevDayMonthPressed: widget.disablePrevDayMonthPressed,
                    disableWeekendPressed: widget.disableWeekendPressed,
                    disableYesterdayPressed: widget.disableYesterdayPressed,
                    firstDayOfWeek: widget.firstDayOfWeek ?? 0,
                    inactiveYesterdays: widget.inactiveYesterdays,
                    index: index,
                    isNextMonthDay: isNextMonthDay,
                    isPrevMonthDay: isPrevMonthDay,
                    isSelectable: isSelectable,
                    isSelectedDay: isSelectedDay,
                    isThisMonthDay: isThisMonthDay,
                    isToday: isToday,
                    isWeekendDay: isWeekendDay,
                    isYesterday: isYesterday,
                    highlightToday: widget.highlightToday,
                    localeDate: _localeDate,
                    now: now,
                    onDayLongPressed: widget.onDayLongPressed,
                    onDayPressed: (date) {
                      setState(() {
                        _selectedDate = date;
                      });
                      widget.onDayPressed?.call(date);
                    },
                    style: widget.style,
                  );
                },
              ),
            ),
          ),
        ],
      ),
    );
  }

  AnimatedBuilder weekBuilder(int slideIndex) {
    final double screenWidth = MediaQuery.of(context).size.width;
    List<DateTime> weekDays = _weeks[slideIndex];

    weekDays = weekDays.map((weekDay) => weekDay.add(Duration(days: firstDayOfWeek))).toList();

    return AnimatedBuilder(
        animation: _controller,
        builder: (context, child) {
          double value = 1.0;
          if (_controller.position.haveDimensions) {
            value = _controller.page! - slideIndex;
            value = (1 - (value.abs() * .5)).clamp(0.0, 1.0);
          }

          return Center(
            child: SizedBox(
              height: Curves.easeOut.transform(value) * CALENDAR_HEIGHT,
              width: Curves.easeOut.transform(value) * screenWidth,
              child: child,
            ),
          );
        },
        child: Stack(
          children: <Widget>[
            Positioned(
              child: Container(
                width: double.infinity,
                height: double.infinity,
                child: GridView.count(
                  physics: widget.customGridViewPhysics,
                  crossAxisCount: 7,
                  childAspectRatio: widget.childAspectRatio,
                  padding: EdgeInsets.zero,
                  children: List.generate(weekDays.length, (index) {
                    /// last day of month + weekday
                    final bool isToday = weekDays[index].day == DateTime.now().day &&
                        weekDays[index].month == DateTime.now().month &&
                        weekDays[index].year == DateTime.now().year;

                    final bool isSelectedDay = _selectedDate.year == weekDays[index].year &&
                        _selectedDate.month == weekDays[index].month &&
                        _selectedDate.day == weekDays[index].day;

                    final bool isPrevMonthDay = weekDays[index].month < _targetDate.month;

                    final bool isNextMonthDay = weekDays[index].month > _targetDate.month;

                    final bool isThisMonthDay = !isPrevMonthDay && !isNextMonthDay;

                    final bool isWeekendDay =
                        weekDays[index].weekday == DateTime.saturday || weekDays[index].weekday == DateTime.sunday;

                    final DateTime now = DateTime(weekDays[index].year, weekDays[index].month, weekDays[index].day);

                    final bool isSelectable = !(now.millisecondsSinceEpoch < minDate.millisecondsSinceEpoch ||
                        now.millisecondsSinceEpoch > maxDate.millisecondsSinceEpoch);

                    final bool isYesterday = now.isBefore(DateTime.now()) && !isToday;

                    return SeniorCalendarDay(
                      disableDayPressed: widget.disableDayPressed,
                      disableNextDayMonthPressed: widget.disableNextDayMonthPressed,
                      disablePrevDayMonthPressed: widget.disablePrevDayMonthPressed,
                      disableWeekendPressed: widget.disableWeekendPressed,
                      disableYesterdayPressed: widget.disableYesterdayPressed,
                      firstDayOfWeek: widget.firstDayOfWeek ?? 0,
                      inactiveYesterdays: widget.inactiveYesterdays,
                      index: index,
                      isNextMonthDay: isNextMonthDay,
                      isPrevMonthDay: isPrevMonthDay,
                      isSelectable: isSelectable,
                      isSelectedDay: isSelectedDay,
                      isThisMonthDay: isThisMonthDay,
                      isToday: isToday,
                      isWeekendDay: isWeekendDay,
                      isYesterday: isYesterday,
                      highlightToday: widget.highlightToday,
                      localeDate: _localeDate,
                      now: now,
                      onDayLongPressed: widget.onDayLongPressed,
                      onDayPressed: (date) {
                        setState(() {
                          _selectedDate = date;
                        });
                        widget.onDayPressed?.call(date);
                      },
                      style: widget.style,
                    );
                  }),
                ),
              ),
            ),
          ],
        ));
  }

  List<DateTime> _getDaysInWeek([DateTime? selectedDate]) {
    selectedDate ??= new DateTime.now();

    final firstDayOfCurrentWeek = _firstDayOfWeek(selectedDate);
    final lastDayOfCurrentWeek = _lastDayOfWeek(selectedDate);

    return _daysInRange(firstDayOfCurrentWeek, lastDayOfCurrentWeek).toList();
  }

  DateTime _firstDayOfWeek(DateTime date) {
    final day = _createUTCMiddayDateTime(date);
    return day.subtract(new Duration(days: date.weekday % 7));
  }

  DateTime _lastDayOfWeek(DateTime date) {
    final day = _createUTCMiddayDateTime(date);
    return day.add(new Duration(days: 7 - day.weekday % 7));
  }

  DateTime _createUTCMiddayDateTime(DateTime date) {
    // Magic const: 12 is to maintain compatibility with date_utils
    return new DateTime.utc(date.year, date.month, date.day, 12, 0, 0);
  }

  Iterable<DateTime> _daysInRange(DateTime start, DateTime end) {
    var offset = start.timeZoneOffset;

    return List<int>.generate(end.difference(start).inDays, (i) => i + 1).map((int i) {
      var d = start.add(Duration(days: i - 1));

      final timeZoneDiff = d.timeZoneOffset - offset;
      if (timeZoneDiff.inSeconds != 0) {
        offset = d.timeZoneOffset;
        d = d.subtract(new Duration(seconds: timeZoneDiff.inSeconds));
      }
      return d;
    });
  }

  Future _selectDateFromPicker() async {
    final DateTime? selected = await showDatePicker(
      context: context,
      initialDate: _selectedDate,
      firstDate: minDate,
      lastDate: maxDate,
    );

    if (selected != null) {
      // updating selected date range based on selected week
      setState(() {
        _selectedDate = selected;
      });
      widget.onDayPressed?.call(selected);
    }
  }

  void _setDatesAndWeeks() {
    // Setup default calendar format
    final List<DateTime> date = [];
    final List<List<DateTime>> week = [];
    int currentDateIndex = 0;

    for (int i = 0;
        0 >= DateTime(minDate.year, minDate.month + i).difference(DateTime(maxDate.year, maxDate.month)).inDays;
        i++,) {
      date.add(DateTime(minDate.year, minDate.month + i, 1));
      if (date.last.difference(DateTime(_targetDate.year, _targetDate.month)).inDays == 0) {
        currentDateIndex = i;
      }
    }

    // Setup week-only format
    for (int i = 0;
        0 >= minDate.add(Duration(days: 7 * i)).difference(maxDate.add(const Duration(days: 7))).inDays;
        i++,) {
      week.add(_getDaysInWeek(minDate.add(new Duration(days: 7 * i))));
    }

    _startWeekday = date[currentDateIndex].weekday - firstDayOfWeek;

    if (_startWeekday == 7) {
      _startWeekday = 0;
    }
    _endWeekday = DateTime(date[currentDateIndex].year, date[currentDateIndex].month + 1, 1).weekday - firstDayOfWeek;
    _dates = date;
    _weeks = week;
  }

  void _setDate([int page = -1]) {
    if (page == -1) {
      setState(() {
        _setDatesAndWeeks();
      });
    } else {
      if (widget.weekFormat) {
        setState(() {
          _pageNum = page;
          _targetDate = _weeks[page].first;
        });

        _controller.animateToPage(page, duration: const Duration(milliseconds: 1), curve: const Threshold(0.0));
      } else {
        setState(() {
          _pageNum = page;
          _targetDate = _dates[page];
          _startWeekday = _dates[page].weekday - firstDayOfWeek;
          _endWeekday = _lastDayOfWeek(_dates[page]).weekday - firstDayOfWeek;
        });
        _controller.animateToPage(page, duration: const Duration(milliseconds: 1), curve: const Threshold(0.0));
      }

      final onCalendarChanged = widget.onCalendarChanged;
      if (onCalendarChanged != null) {
        WidgetsBinding.instance.addPostFrameCallback((_) {
          onCalendarChanged(!widget.weekFormat ? _dates[page] : _weeks[page][firstDayOfWeek]);
        });
      }
    }
  }

  String buildHeaderText() {
    if (widget.headerText != null) {
      return widget.headerText!;
    } else {
      if (widget.weekFormat) {
        return '${_localeDate.format(_weeks[_pageNum].first)}';
      } else {
        final text = _localeDate.format(_dates[_pageNum]);
        return '${text[0].toUpperCase()}${text.substring(1).toLowerCase()}';
      }
    }
  }
}
